//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Diagnostics;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents the base for all specialized axis renderer. Initialization common too all
  /// axis renderer should come here.
  /// </summary>
  internal abstract class AxisRenderer : Renderer
  {
    /// <summary>
    /// Initializes a new instance of the AxisRenderer class with the specified renderer parameters.
    /// </summary>
    internal AxisRenderer(RendererParameters parms) : base(parms)
    {
    }

    /// <summary>
    /// Initializes the axis title of the rendererInfo. All missing font attributes will be taken
    /// from the specified defaultFont.
    /// </summary>
    protected void InitAxisTitle(AxisRendererInfo rendererInfo, XFont defaultFont)
    {
      if (rendererInfo.axis.title != null)
      {
        rendererInfo.AxisTitleAlignment = rendererInfo.axis.title.alignment;
        rendererInfo.AxisTitleFont = RendererInfo.XFontFromFont(rendererInfo.axis.title.font, defaultFont);
        XColor fontColor = XColors.Black;
        if (rendererInfo.axis.title.font != null && !rendererInfo.axis.title.font.color.IsEmpty)
          fontColor = rendererInfo.axis.title.font.color;
        rendererInfo.AxisTitleBrush = new XSolidBrush(fontColor);
        rendererInfo.AxisTitleOrientation = rendererInfo.axis.title.orientation;
      }
    }

    /// <summary>
    /// Initializes the tick labels of the rendererInfo. All missing font attributes will be taken
    /// from the specified defaultFont.
    /// </summary>
    protected void InitTickLabels(AxisRendererInfo rendererInfo, XFont defaultFont)
    {
      if (rendererInfo.axis.tickLabels != null)
      {
        rendererInfo.TickLabelsFont = RendererInfo.XFontFromFont(rendererInfo.axis.tickLabels.font, defaultFont);
        XColor fontColor = XColors.Black;
        if (rendererInfo.axis.tickLabels.font != null &&
          !rendererInfo.axis.tickLabels.font.color.IsEmpty)
          fontColor = rendererInfo.axis.tickLabels.font.color;
        rendererInfo.TickLabelsBrush = new XSolidBrush(fontColor);

        rendererInfo.TickLabelsFormat = rendererInfo.axis.tickLabels.format;
        if (rendererInfo.TickLabelsFormat == null)
          rendererInfo.TickLabelsFormat = "0";
      }
      else
      {
        rendererInfo.TickLabelsFont = defaultFont;
        rendererInfo.TickLabelsBrush = new XSolidBrush(XColors.Black);
        rendererInfo.TickLabelsFormat = "0"; //TODO Unterschiedlich fr X- und Y-Achse
      }
    }
    
    /// <summary>
    /// Initializes the line format of the rendererInfo.
    /// </summary>
    protected void InitAxisLineFormat(AxisRendererInfo rendererInfo)
    {
      if (rendererInfo.axis.lineFormat != null)
      {
        rendererInfo.LineFormat = rendererInfo.axis.lineFormat.Clone();
        if (rendererInfo.LineFormat.color.IsEmpty)
          rendererInfo.LineFormat.color = XColors.Black;
      }
      else
      {
        rendererInfo.LineFormat = new LineFormat();
        rendererInfo.LineFormat.Color = XColors.Black;
        rendererInfo.LineFormat.Visible = true;
        rendererInfo.LineFormat.Width = 2;
      }
    }
    
    /// <summary>
    /// Initializes the gridlines of the rendererInfo.
    /// </summary>
    protected void InitGridlines(AxisRendererInfo rendererInfo)
    {
      if (rendererInfo.axis.minorGridlines != null)
      {
        rendererInfo.MinorGridlinesLineFormat = rendererInfo.axis.minorGridlines.lineFormat;
        if (rendererInfo.axis.HasMinorGridlines)
        {
          if (rendererInfo.MinorGridlinesLineFormat == null)
            rendererInfo.MinorGridlinesLineFormat = new LineFormat();
          else
            rendererInfo.MinorGridlinesLineFormat = rendererInfo.MinorGridlinesLineFormat.Clone();
        }
        if (rendererInfo.MinorGridlinesLineFormat != null)
          InitLineFormat(rendererInfo.MinorGridlinesLineFormat);
      }
      else if (rendererInfo.axis.hasMinorGridlines)
      {
        // No minor gridlines object are given, but user asked for.
        rendererInfo.MinorGridlinesLineFormat = new LineFormat();
        InitLineFormat(rendererInfo.MinorGridlinesLineFormat);
      }

      if (rendererInfo.axis.majorGridlines != null)
      {
        rendererInfo.MajorGridlinesLineFormat = rendererInfo.axis.majorGridlines.lineFormat;
        if (rendererInfo.axis.HasMajorGridlines)
        {
          if (rendererInfo.MajorGridlinesLineFormat == null)
            rendererInfo.MajorGridlinesLineFormat = new LineFormat();
          else
            rendererInfo.MajorGridlinesLineFormat = rendererInfo.MajorGridlinesLineFormat.Clone();
        }
        if (rendererInfo.MajorGridlinesLineFormat != null)
          InitLineFormat(rendererInfo.MajorGridlinesLineFormat);
      }
      else if (rendererInfo.axis.hasMajorGridlines)
      {
        // No major gridlines object are given, but user asked for.
        rendererInfo.MajorGridlinesLineFormat = new LineFormat();
        InitLineFormat(rendererInfo.MajorGridlinesLineFormat);
      }
    }

    /// <summary>
    /// Initializes the line format.
    /// </summary>
    protected void InitLineFormat(LineFormat lineFormat)
    {
      if (lineFormat.color.IsEmpty)
        lineFormat.Color = XColors.Black;
    }

    /// <summary>
    /// Formats string number by string format and returns the result. If string number does not
    /// contain a valid number it will be returned unchanged.
    /// </summary>
    protected string GetFormattedNumber(string number, string format)
    {
      string fmtNumber = number;
      try
      {
        double d = double.Parse(number);
        fmtNumber = d.ToString(format);
      }
      catch (FormatException /*ex*/)
      {
        // it's not a number
      }
      return fmtNumber;
    }
  }
}
