//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents a column chart renderer.
  /// </summary>
  internal class ColumnChartRenderer : ChartRenderer
  {
    /// <summary>
    /// Initializes a new instance of the ColumnChartRenderer class with the
    /// specified renderer parameters.
    /// </summary>
    internal ColumnChartRenderer(RendererParameters parms) : base(parms)
    {
    }

    /// <summary>
    /// Returns an initialized and renderer specific rendererInfo.
    /// </summary>
    internal override RendererInfo Init()
    {
      ChartRendererInfo cri = new ChartRendererInfo();
      cri.chart = (Chart)this.rendererParms.DrawingItem;
      this.rendererParms.RendererInfo = cri;

      InitSeries(cri);

      AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
      cri.xAxisRendererInfo = (AxisRendererInfo)xar.Init();

      AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
      cri.yAxisRendererInfo = (AxisRendererInfo)yar.Init();

      PlotArea plotArea = cri.chart.PlotArea;
      ColumnPlotAreaRenderer renderer = new ColumnPlotAreaRenderer(this.rendererParms);
      cri.plotAreaRendererInfo = (PlotAreaRendererInfo)renderer.Init();

      return cri;
    }
    
    /// <summary>
    /// Layouts and calculates the space used by the line chart.
    /// </summary>
    internal override void Format()
    {
      ChartRendererInfo cri = this.rendererParms.RendererInfo;

      //TODO LegendRenderer

      // axes
      AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
      xar.Format();

      AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
      yar.Format();

      ColumnPlotAreaRenderer renderer = new ColumnPlotAreaRenderer(this.rendererParms);
      renderer.Format();

      // Calculate rects and positions.
      cri.xAxisRendererInfo.X = this.rendererParms.Box.Left + cri.yAxisRendererInfo.Width;
      cri.xAxisRendererInfo.Y = this.rendererParms.Box.Bottom - cri.xAxisRendererInfo.Height;
      cri.xAxisRendererInfo.Width = this.rendererParms.Box.Width - cri.yAxisRendererInfo.Width;
      cri.yAxisRendererInfo.X = this.rendererParms.Box.Left;
      cri.yAxisRendererInfo.Y = this.rendererParms.Box.Top;
      cri.yAxisRendererInfo.Height = this.rendererParms.Box.Height - cri.xAxisRendererInfo.Height;
      cri.plotAreaRendererInfo.X = cri.xAxisRendererInfo.X;
      cri.plotAreaRendererInfo.Y = cri.yAxisRendererInfo.Y;
      cri.plotAreaRendererInfo.Width = cri.xAxisRendererInfo.Width;
      cri.plotAreaRendererInfo.Height = cri.yAxisRendererInfo.Height;
    }

    /// <summary>
    /// Draws the column chart.
    /// </summary>
    internal override void Draw()
    {
      //TODO Legende zeichnen

      //TODO Wall zeichnen (PlotArea-Hintergrund)

      GridlinesRenderer glr = new ColumnLikeGridlinesRenderer(this.rendererParms);
      glr.Draw();

      PlotAreaBorderRenderer pabr = new PlotAreaBorderRenderer(this.rendererParms);
      pabr.Draw();

      ColumnPlotAreaRenderer renderer = new ColumnPlotAreaRenderer(this.rendererParms);
      renderer.Draw();

      if (this.rendererParms.RendererInfo.xAxisRendererInfo.axis != null)
      {
        AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
        xar.Draw();
      }

      if (this.rendererParms.RendererInfo.yAxisRendererInfo.axis != null)
      {
        AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
        yar.Draw();
      }
    }

    /// <summary>
    /// Initializes all necessary data to draw a series for a column chart.
    /// </summary>
    private void InitSeries(ChartRendererInfo rendererInfo)
    {
      SeriesCollection seriesColl = rendererInfo.chart.SeriesCollection;
      rendererInfo.seriesRendererInfos = new SeriesRendererInfo[seriesColl.Count];
      for (int idx = 0; idx < seriesColl.Count; ++idx)
      {
        SeriesRendererInfo sri = new SeriesRendererInfo();
        rendererInfo.seriesRendererInfos[idx] = sri;
        sri.series = seriesColl[idx];

        LineFormat lineFormat;
        if (sri.series.lineFormat != null)
          lineFormat = sri.series.lineFormat.Clone();
        else
          lineFormat = new LineFormat();

        if (lineFormat.color.IsEmpty)
          lineFormat.Color = XColor.FromKnownColor(System.Drawing.KnownColor.Black);

        FillFormat fillFormat;
        if (sri.series.fillFormat != null)
          fillFormat = sri.series.fillFormat.Clone();
        else
          fillFormat = new FillFormat();

        if (fillFormat.color.IsEmpty)
          fillFormat.Color = ColumnColors.Item(idx);

        sri.pointRendererInfos = new PointRendererInfo[sri.series.seriesElements.Count];
        for (int pointIdx = 0; pointIdx < sri.pointRendererInfos.Length; ++pointIdx)
        {
          PointRendererInfo pri = new PointRendererInfo();
          Point point = sri.series.seriesElements[pointIdx];
          pri.point = point;
          if (point != null)
          {
            pri.LineFormat = lineFormat;
            pri.FillFormat = fillFormat;
            if (point.lineFormat != null)
              pri.LineFormat = point.lineFormat;
            if (point.fillFormat != null)
              pri.FillFormat = point.fillFormat;
          }
          sri.pointRendererInfos[pointIdx] = pri;
        }
      }
    }
  }
}