//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Diagnostics;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents a plot area renderer of clustered columns, i. e. all columns are drawn side by side.
  /// </summary>
  internal class ColumnPlotAreaRenderer : PlotAreaRenderer
  {
    /// <summary>
    /// Represents one data point of a series and the corresponding rectangle.
    /// </summary>
    private class ColumnDescription
    {
      public PointRendererInfo rendererInfo;
      public XRect Rect;
    }

    /// <summary>
    /// Initializes a new instance of the ColumnPlotAreaRenderer class with the
    /// specified renderer parameters.
    /// </summary>
    internal ColumnPlotAreaRenderer(RendererParameters parms) : base(parms)
    {
    }

    /// <summary>
    /// Returns an initialized PlotAreaRendererInfo.
    /// </summary>
    internal override RendererInfo Init()
    {
      PlotAreaRendererInfo pari = new PlotAreaRendererInfo();
      pari.plotArea = this.rendererParms.RendererInfo.chart.plotArea;
      InitLineFormat(pari);
      return pari;
    }

    /// <summary>
    /// Draws the content of the column plot area.
    /// </summary>
    internal override void Draw()
    {
      XRect plotAreaBox = this.rendererParms.RendererInfo.plotAreaRendererInfo.Rect;
      if (plotAreaBox.IsEmpty)
        return;

      XGraphics gfx = this.rendererParms.Graphics;
      ChartRendererInfo cri = this.rendererParms.RendererInfo;

      double xMin = cri.xAxisRendererInfo.MinimumScale;
      double xMax = cri.xAxisRendererInfo.MaximumScale;
      double yMin = cri.yAxisRendererInfo.MinimumScale;
      double yMax = cri.yAxisRendererInfo.MaximumScale;

      this.matrix = cri.PlotAreaMatrix;
      LineFormatRenderer lineFormatRenderer;

      // Under some circumstances it is possible that no zero base line will be drawn,
      // e. g. because of unfavourable minimum/maximum scale and/or major tick, so force to draw
      // a zero base line if necessary.
      if (cri.yAxisRendererInfo.MajorGridlinesLineFormat != null ||
          cri.yAxisRendererInfo.MinorGridlinesLineFormat != null)
      {
        if (yMin < 0 && yMax > 0)
        {
          XPoint[] points = new XPoint[2];
          points[0].X = xMin;
          points[0].Y = 0;
          points[1].X = xMax;
          points[1].Y = 0;
          this.matrix.TransformPoints(points);

          if (cri.yAxisRendererInfo.MinorGridlinesLineFormat != null)
            lineFormatRenderer = new LineFormatRenderer(gfx, cri.yAxisRendererInfo.MinorGridlinesLineFormat);
          else
            lineFormatRenderer = new LineFormatRenderer(gfx, cri.yAxisRendererInfo.MajorGridlinesLineFormat);

          lineFormatRenderer.DrawLine(points[0], points[1]);
        }
      }

      // 1. calculate positions
      ColumnDescription[] columns = CalcColumns();

      // 2. draw column
      XGraphicsState state = gfx.Save();
      // BUG in PDFsharp: treat XPen(XColor.Empty) as nothing, same to XSolidBrush.
      XPen pen = new XPen(XColor.Empty);
      foreach (ColumnDescription column in columns)
      {
        // Do not draw column if value is outside yMin/yMax range. Clipping does not make sense.
        if (IsDataInside(yMin, yMax, column.rendererInfo.point.value))
        {
          XBrush brush = new XSolidBrush(column.rendererInfo.FillFormat.Color);
          gfx.DrawRectangle(brush, column.Rect);
        }
      }

      // 3. draw borders around column.
      // A border can overlap neighbor columns, so it is important to draw borders at the end.
      foreach (ColumnDescription column in columns)
      {
        // Do not draw column if value is outside yMin/yMax range. Clipping does not make sense.
        if (IsDataInside(yMin, yMax, column.rendererInfo.point.value))
        {
          lineFormatRenderer = new LineFormatRenderer(gfx, column.rendererInfo.LineFormat);
          lineFormatRenderer.DrawRectangle(column.Rect);
        }
      }
      gfx.Restore(state);
    }

    /// <summary>
    /// Calculates the position, width and height of each column of all series.
    /// </summary>
    private ColumnDescription[] CalcColumns()
    {
      ChartRendererInfo cri = this.rendererParms.RendererInfo;
      if (cri.seriesRendererInfos.Length == 0)
        return null;

      double xMin = cri.xAxisRendererInfo.MinimumScale;
      double yMin = cri.yAxisRendererInfo.MinimumScale;
      double yMax = cri.yAxisRendererInfo.MaximumScale;

      int pointCount = 0;
      foreach (SeriesRendererInfo sr in cri.seriesRendererInfos)
        pointCount += sr.series.seriesElements.Count;

      // Space shared by one clustered column.
      double groupWidth = cri.xAxisRendererInfo.MajorTick;

      // Space used by one column.
      double columnWidth = groupWidth * 3 / 4 / cri.seriesRendererInfos.Length;

      ColumnDescription[] cds = new ColumnDescription[pointCount];

      int pointIdx = 0;
      int seriesIdx = 0;
      XPoint[] points = new XPoint[2];
      foreach (SeriesRendererInfo sri in cri.seriesRendererInfos)
      {
        // Set x to first clustered column for each series.
        double x = xMin + groupWidth / 2;
        
        // Offset for columns of a particular series from the start of a clustered cloumn.
        double dx = (columnWidth * seriesIdx) - (columnWidth / 2 * cri.seriesRendererInfos.Length);

        foreach (PointRendererInfo pri in sri.pointRendererInfos)
        {
          ColumnDescription cd = new ColumnDescription();
          cd.rendererInfo = pri;

          if (pri.point != null)
          {
            double x0 = x + dx;
            double x1 = x + dx + columnWidth;
            double y0 = yMin;
            double y1 = pri.point.Value;

            // Draw from zero base line, if it exists.
            if (y0 < 0 && yMax >= 0)
              y0 = 0;

            // y0 should always be lower than y1, i. e. draw column from bottom to top.
            if (y1 < 0 && y1 < y0)
            {
              double y = y0;
              y0 = y1;
              y1 = y;
            }

            points[0].X = x0; // upper left
            points[0].Y = y1;
            points[1].X = x1; // lower right
            points[1].Y = y0;

            this.matrix.TransformPoints(points);

            cd.Rect = new XRect(points[0].X,
                                points[0].Y,
                                points[1].X - points[0].X,
                                points[1].Y - points[0].Y);
          }
          cds[pointIdx] = cd;
          ++pointIdx;
          x++; // Next clustered column.
        }
        seriesIdx++;
      }
      return cds;
    }

    /// <summary>
    /// If yValue is within the range from yMin to yMax returns true, otherwise false.
    /// </summary>
    private bool IsDataInside(double yMin, double yMax, double yValue)
    {
      return yValue <= yMax && yValue >= yMin;
    }

    /// <summary>
    /// Matrix used to map data points into plot area space.
    /// </summary>
    XMatrix matrix;
  }
}
