//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents a line chart renderer.
  /// </summary>
  internal class LineChartRenderer : ChartRenderer
  {
    /// <summary>
    /// Initializes a new instance of the LineChartRenderer class with the specified renderer parameters.
    /// </summary>
    internal LineChartRenderer(RendererParameters parms) : base(parms)
    {
    }

    /// <summary>
    /// Returns an initialized and renderer specific rendererInfo.
    /// </summary>
    internal override RendererInfo Init()
    {
      ChartRendererInfo cri = new ChartRendererInfo();
      cri.chart = (Chart)this.rendererParms.DrawingItem;
      this.rendererParms.RendererInfo = cri;

      InitSeries(cri);

      AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
      cri.xAxisRendererInfo = (AxisRendererInfo)xar.Init();

      AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
      cri.yAxisRendererInfo = (AxisRendererInfo)yar.Init();

      PlotArea plotArea = cri.chart.PlotArea;
      LinePlotAreaRenderer lpar = new LinePlotAreaRenderer(this.rendererParms);
      cri.plotAreaRendererInfo = (PlotAreaRendererInfo)lpar.Init();

      return cri;
    }

    /// <summary>
    /// Layouts and calculates the space used by the line chart.
    /// </summary>
    internal override void Format()
    {
      ChartRendererInfo cri = this.rendererParms.RendererInfo;

      //TODO LegendRenderer

      // axes
      AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
      xar.Format();

      AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
      yar.Format();

      LinePlotAreaRenderer lpar = new LinePlotAreaRenderer(this.rendererParms);
      lpar.Format();

      // calculate rects and positions
      cri.xAxisRendererInfo.X = this.rendererParms.Box.Left + cri.yAxisRendererInfo.Width;
      cri.xAxisRendererInfo.Y = this.rendererParms.Box.Bottom - cri.xAxisRendererInfo.Height;
      cri.xAxisRendererInfo.Width = this.rendererParms.Box.Width - cri.yAxisRendererInfo.Width;
      cri.yAxisRendererInfo.X = this.rendererParms.Box.Left;
      cri.yAxisRendererInfo.Y = this.rendererParms.Box.Top;
      cri.yAxisRendererInfo.Height = this.rendererParms.Box.Height - cri.xAxisRendererInfo.Height;
      cri.plotAreaRendererInfo.X = cri.xAxisRendererInfo.X;
      cri.plotAreaRendererInfo.Y = cri.yAxisRendererInfo.Y;
      cri.plotAreaRendererInfo.Width = cri.xAxisRendererInfo.Width;
      cri.plotAreaRendererInfo.Height = cri.yAxisRendererInfo.Height;
    }

    /// <summary>
    /// Draws the line chart.
    /// </summary>
    internal override void Draw()
    {
      //TODO Legende zeichnen

      //TODO Wall zeichnen (PlotArea-Hintergrund)

      // Draw gridlines.
      GridlinesRenderer glr = new ColumnLikeGridlinesRenderer(this.rendererParms);
      glr.Draw();

      PlotAreaBorderRenderer pabr = new PlotAreaBorderRenderer(this.rendererParms);
      pabr.Draw();

      // Draw line chart's plot area.
      LinePlotAreaRenderer lpar = new LinePlotAreaRenderer(this.rendererParms);
      lpar.Draw();

      // Draw x- and y-axis.
      if (this.rendererParms.RendererInfo.xAxisRendererInfo.axis != null)
      {
        AxisRenderer xar = new HorizontalXAxisRenderer(this.rendererParms);
        xar.Draw();
      }

      if (this.rendererParms.RendererInfo.yAxisRendererInfo.axis != null)
      {
        AxisRenderer yar = new VerticalYAxisRenderer(this.rendererParms);
        yar.Draw();
      }
    }

    /// <summary>
    /// Initializes all necessary data to draw a series for a line chart.
    /// </summary>
    private void InitSeries(ChartRendererInfo rendererInfo)
    {
      SeriesCollection seriesColl = rendererInfo.chart.SeriesCollection;
      rendererInfo.seriesRendererInfos = new SeriesRendererInfo[seriesColl.Count];
      for (int idx = 0; idx < seriesColl.Count; ++idx)
      {
        SeriesRendererInfo sri = new SeriesRendererInfo();
        rendererInfo.seriesRendererInfos[idx] = sri;
        sri.series = seriesColl[idx];

        LineFormat lineFormat;
        if (sri.series.lineFormat != null)
          lineFormat = sri.series.lineFormat.Clone();
        else
          lineFormat = new LineFormat();
        if (lineFormat.color.IsEmpty)
          lineFormat.Color = LineColors.Item(idx);
        sri.LineFormat = lineFormat;
        
        sri.MarkerForegroundColor = sri.series.markerForegroundColor;
        if (sri.MarkerForegroundColor.IsEmpty)
          sri.MarkerForegroundColor = lineFormat.color;

        sri.MarkerBackgroundColor = sri.series.markerBackgroundColor;
        if (sri.MarkerBackgroundColor.IsEmpty)
          sri.MarkerBackgroundColor = lineFormat.color;

        sri.MarkerSize = sri.series.markerSize;
        if (sri.MarkerSize == 0)
          sri.MarkerSize = 7;

        // MarkerStyle for each series was initialized in SeriesCollection.AddSeries.
        sri.MarkerStyle = sri.series.markerStyle;
      }
    }
  }
}
