//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents a pie chart renderer.
  /// </summary>
  internal abstract class PieChartRenderer : ChartRenderer
  {
    /// <summary>
    /// Initializes a new instance of the PieChartRenderer class with the
    /// specified renderer parameters.
    /// </summary>
    internal PieChartRenderer(RendererParameters parms) : base(parms)
    {
    }

    /// <summary>
    /// Factory method which returns the specific plot area renderer.
    /// </summary>
    protected abstract PlotAreaRenderer GetPlotAreaRenderer(RendererParameters parms);

    /// <summary>
    /// Returns an initialized and renderer specific rendererInfo.
    /// </summary>
    internal override RendererInfo Init()
    {
      ChartRendererInfo cri = new ChartRendererInfo();
      cri.chart = (Chart)this.rendererParms.DrawingItem;
      this.rendererParms.RendererInfo = cri;

      InitSeries(cri);

      PlotArea plotArea = cri.chart.PlotArea;
      PlotAreaRenderer renderer = GetPlotAreaRenderer(this.rendererParms);
      cri.plotAreaRendererInfo = (PlotAreaRendererInfo)renderer.Init();

      return cri;
    }
    
    /// <summary>
    /// Layouts and calculates the space used by the pie chart.
    /// </summary>
    internal override void Format()
    {
      ChartRendererInfo cri = this.rendererParms.RendererInfo;

      //TODO LegendRenderer

      PlotAreaRenderer renderer = GetPlotAreaRenderer(this.rendererParms);
      renderer.Format();

      // Calculate rects and positions.
      cri.plotAreaRendererInfo.Rect = this.rendererParms.Box;
      if (this.rendererParms.Box.Width != this.rendererParms.Box.Height)
      {
        double edge = Math.Min(this.rendererParms.Box.Width, this.rendererParms.Box.Height);
        cri.plotAreaRendererInfo.X += (this.rendererParms.Box.Width - edge) / 2;
        cri.plotAreaRendererInfo.Y += (this.rendererParms.Box.Height - edge) / 2;
        cri.plotAreaRendererInfo.Width = edge;
        cri.plotAreaRendererInfo.Height = edge;
      }
    }

    /// <summary>
    /// Draws the pie chart.
    /// </summary>
    internal override void Draw()
    {
      //TODO Legende zeichnen

      //TODO Wall zeichnen (PlotArea-Hintergrund)

      PlotAreaBorderRenderer pabr = new PlotAreaBorderRenderer(this.rendererParms);
      pabr.Draw();

      PlotAreaRenderer renderer = GetPlotAreaRenderer(this.rendererParms);
      renderer.Draw();
    }

    /// <summary>
    /// Initializes all necessary data to draw a series for a pie chart.
    /// </summary>
    protected void InitSeries(ChartRendererInfo rendererInfo)
    {
      SeriesCollection seriesColl = rendererInfo.chart.SeriesCollection;
      rendererInfo.seriesRendererInfos = new SeriesRendererInfo[seriesColl.Count];
      for (int idx = 0; idx < seriesColl.Count; ++idx)
      {
        SeriesRendererInfo sri = new SeriesRendererInfo();
        rendererInfo.seriesRendererInfos[idx] = sri;
        sri.series = seriesColl[idx];

        LineFormat lineFormat;
        if (sri.series.lineFormat != null)
          lineFormat = sri.series.lineFormat.Clone();
        else
          lineFormat = new LineFormat();

        if (lineFormat.color.IsEmpty)
          lineFormat.Color = XColor.FromKnownColor(System.Drawing.KnownColor.Black);
        sri.LineFormat = lineFormat;

        FillFormat fillFormat;
        if (sri.series.fillFormat != null)
          fillFormat = sri.series.fillFormat.Clone();
        else
          fillFormat = new FillFormat();

        if (fillFormat.color.IsEmpty)
          fillFormat.Color = ColumnColors.Item(idx);
        sri.Fillformat = fillFormat;

        sri.pointRendererInfos = new PointRendererInfo[sri.series.seriesElements.Count];
        for (int pointIdx = 0; pointIdx < sri.pointRendererInfos.Length; ++pointIdx)
        {
          PointRendererInfo pri = new PointRendererInfo();
          Point point = sri.series.seriesElements[pointIdx];
          pri.point = point;
          if (point != null)
          {
            pri.LineFormat = lineFormat;
            if (point.lineFormat != null)
              pri.LineFormat = point.lineFormat;
            if (point.fillFormat != null)
              pri.FillFormat = point.fillFormat;
            else
            {
              pri.FillFormat = new FillFormat();
              pri.FillFormat.color = PieColors.Item(pointIdx);
            }
          }
          sri.pointRendererInfos[pointIdx] = pri;
        }
      }
    }
  }
}
