//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents a exploded pie plot area renderer.
  /// </summary>
  internal class PieExplodedPlotAreaRenderer : PiePlotAreaRenderer
	{
    /// <summary>
    /// Initializes a new instance of the PieExplodedPlotAreaRenderer class
    /// with the specified renderer parameters.
    /// </summary>
    internal PieExplodedPlotAreaRenderer(RendererParameters parms) : base(parms)
		{
		}

    /// <summary>
    /// Calculate angles for each sector.
    /// </summary>
    protected override SectorDescription[] CalcSectors()
    {
      ChartRendererInfo cri = this.rendererParms.RendererInfo;
      if (cri.seriesRendererInfos.Length == 0)
        return null;

      SeriesRendererInfo sri = cri.seriesRendererInfos[0];

      double sumValues = 0;
      int sectorCount = 0;
      foreach (PointRendererInfo pri in sri.pointRendererInfos)
      {
        if (!double.IsNaN(pri.point.value))
        {
          sumValues += Math.Abs(pri.point.value);
          ++sectorCount;
        }
      }

      SectorDescription[] sectors = new SectorDescription[sectorCount];
      if (sectorCount == 0)
        return sectors;

      XRect rect = cri.plotAreaRendererInfo.Rect;
      XPoint origin = new XPoint(rect.X + rect.Width / 2, rect.Y + rect.Height / 2);
      XRect innerRect = XRect.Empty;
      XPoint p1 = XPoint.Empty;

      double midAngle = 0, sectorStartAngle = 0, sectorSweepAngle = 0,
             deltaAngle = 2, startAngle = 270, sweepAngle = 0,
             rInnerCircle = rect.Width / 15,
             rOuterCircle = rect.Width / 2;

      int sectorIndex = 0;
      foreach (PointRendererInfo pri in sri.pointRendererInfos)
      {
        if (!double.IsNaN(pri.point.value))
        {
          sweepAngle = 360 / (sumValues / Math.Abs(pri.point.value));

          midAngle = startAngle + sweepAngle / 2;
          sectorStartAngle = Math.Max(0, startAngle + deltaAngle);
          sectorSweepAngle = Math.Max(sweepAngle, sweepAngle - deltaAngle);

          p1.X = origin.X + rInnerCircle * Math.Cos(midAngle / 180 * Math.PI);
          p1.Y = origin.Y + rInnerCircle * Math.Sin(midAngle / 180 * Math.PI);
          innerRect.X = p1.X - rOuterCircle + rInnerCircle;
          innerRect.Y = p1.Y - rOuterCircle + rInnerCircle;
          innerRect.Width = (rOuterCircle - rInnerCircle) * 2;
          innerRect.Height = innerRect.Width;

          sectors[sectorIndex].rendererInfo = pri;
          sectors[sectorIndex].rect = innerRect;
          sectors[sectorIndex].startAngle = sectorStartAngle;
          sectors[sectorIndex].sweepAngle = sectorSweepAngle;
          ++sectorIndex;

          startAngle += sweepAngle;
        }
      }

      return sectors;
    }
  }
}
