//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents the base for all pie plot area renderer.
  /// </summary>
  internal abstract class PiePlotAreaRenderer : PlotAreaRenderer
	{
    /// <summary>
    /// Represents one sector of a series used by a pie chart.
    /// </summary>
    protected struct SectorDescription
    {
      public PointRendererInfo rendererInfo;

      public XRect rect;
      public double startAngle;
      public double sweepAngle;
    };

    /// <summary>
    /// Initializes a new instance of the PiePlotAreaRenderer class
    /// with the specified renderer parameters.
    /// </summary>
    internal PiePlotAreaRenderer(RendererParameters parms) : base(parms)
		{
		}

    /// <summary>
    /// Returns an initialized PlotAreaRendererInfo.
    /// </summary>
    internal override RendererInfo Init()
    {
      PlotAreaRendererInfo pari = new PlotAreaRendererInfo();
      pari.plotArea = this.rendererParms.RendererInfo.chart.plotArea;
      InitLineFormat(pari);
      return pari;
    }

    /// <summary>
    /// Draws the content of the pie plot area.
    /// </summary>
    internal override void Draw()
    {
      XRect plotAreaRect = this.rendererParms.RendererInfo.plotAreaRendererInfo.Rect;
      if (plotAreaRect.IsEmpty)
        return;

      // 1. Calculate sectors.
      SectorDescription[] sectors = CalcSectors();

      // 2. Draw sectors.
      XPen pen = new XPen(XColor.Empty);
      XBrush brush;
      XGraphics gfx = this.rendererParms.Graphics;
      foreach (SectorDescription sector in sectors)
      {
        brush = new XSolidBrush(sector.rendererInfo.FillFormat.Color);
        gfx.DrawPie(pen, brush, sector.rect, sector.startAngle, sector.sweepAngle);
      }

      // 3. Draw border of the sectors.
      brush = new XSolidBrush(XColor.Empty);
      foreach (SectorDescription sector in sectors)
      {
        pen = new XPen(sector.rendererInfo.LineFormat.Color);
        gfx.DrawPie(pen, brush, sector.rect, sector.startAngle, sector.sweepAngle);
      }
    }

    /// <summary>
    /// Calculates the specific positions for each sector.
    /// </summary>
    protected abstract SectorDescription[] CalcSectors();
  }
}
