//
// PDFsharp.Charting - A charting library based on PDFsharp
//
// Authors:
//   Niklas Schneider (mailto:Niklas.Schneider@pdfsharp.com)
//
// Copyright (c) 2005 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using PdfSharp.Drawing;

namespace PdfSharp.Charting.Renderers
{
  /// <summary>
  /// Represents the base class of all renderer infos.
  /// Renderer infos are used to hold all necessary information and time consuming calculations
  /// between rendering cycles.
  /// </summary>
  internal abstract class RendererInfo
  {
    internal static XFont XFontFromFont(Font font, XFont defaultFont)
    {
      XFont xfont = defaultFont;
      if (font != null)
      {
        string fontFamily = font.name;
        if (fontFamily == "")
          fontFamily = defaultFont.Name;

        XFontStyle fontStyle = defaultFont.Style;
        if (font.bold)
          fontStyle |= XFontStyle.Bold;
        if (font.italic)
          fontStyle |= XFontStyle.Italic;

        double size = font.size.Point; //emSize???
        if (size == 0)
          size = defaultFont.Size;

        xfont = new XFont(fontFamily, size, fontStyle);
      }
      return xfont;
    }
  }

  /// <summary>
  /// Base class for all renderer infos which defines an area.
  /// </summary>
  internal abstract class AreaRendererInfo : RendererInfo
  {
    /// <summary>
    /// Gets or sets the x coordinate of this rectangle.
    /// </summary>
    internal double X
    {
      get {return this.rect.X;}
      set {this.rect.X = value;}
    }

    /// <summary>
    /// Gets or sets the y coordinate of this rectangle.
    /// </summary>
    internal double Y
    {
      get {return this.rect.Y;}
      set {this.rect.Y = value;}
    }

    /// <summary>
    /// Gets or sets the width of this rectangle.
    /// </summary>
    internal double Width
    {
      get {return this.rect.Width;}
      set {this.rect.Width = value;}
    }

    /// <summary>
    /// Gets or sets the height of this rectangle.
    /// </summary>
    internal double Height
    {
      get {return this.rect.Height;}
      set {this.rect.Height = value;}
    }

    /// <summary>
    /// Gets the area's rectangle.
    /// </summary>
    internal XRect Rect
    {
      get {return this.rect;}
      set {this.rect = value;}
    }
    XRect rect;
  }

  /// <summary>
  /// A ChartRendererInfo stores information of all main parts of a chart like axis renderer info or
  /// plotarea renderer info.
  /// </summary>
  internal class ChartRendererInfo : AreaRendererInfo
  {
    internal Chart chart;

    internal AxisRendererInfo xAxisRendererInfo;
    internal AxisRendererInfo yAxisRendererInfo;
    //internal AxisRendererInfo zAxisRendererInfo; // not yet used
    internal PlotAreaRendererInfo plotAreaRendererInfo;
    internal SeriesRendererInfo[] seriesRendererInfos;

    /// <summary>
    /// Gets the chart's default font for rendering.
    /// </summary>
    internal XFont DefaultFont
    {
      get
      {
        if (defaultFont == null)
          defaultFont = XFontFromFont(this.chart.font, new XFont("Arial", 12, XFontStyle.Regular));

        return defaultFont;
      }
    }
    XFont defaultFont;

    /// <summary>
    /// Gets the plot area's matrix.
    /// </summary>
    internal XMatrix PlotAreaMatrix //Rename ColumnLike... / Horizontal... ???
    {
      get
      {
        if (plotAreaMatrix.IsIdentity)
        {
          double xMin = this.xAxisRendererInfo.MinimumScale;
          double xMax = this.xAxisRendererInfo.MaximumScale;
          double yMin = this.yAxisRendererInfo.MinimumScale;
          double yMax = this.yAxisRendererInfo.MaximumScale;

          XRect plotAreaBox = this.plotAreaRendererInfo.Rect;
          plotAreaMatrix.Translate(-xMin, yMax);
          plotAreaMatrix.Scale(plotAreaBox.Width / xMax, plotAreaBox.Height / (yMax - yMin), XMatrixOrder.Append);
          plotAreaMatrix.Scale(1, -1);
          plotAreaMatrix.Translate(plotAreaBox.X, plotAreaBox.Y, XMatrixOrder.Append);
        }
        return plotAreaMatrix;
      }
    }
    XMatrix plotAreaMatrix = XMatrix.Identity;
  }

  /// <summary>
  /// PointRendererInfo is used to render one single data point which is part of a data series.
  /// </summary>
  internal class PointRendererInfo : RendererInfo
  {
    internal Point point;

    internal LineFormat LineFormat;
    internal FillFormat FillFormat;
  }

  /// <summary>
  /// SeriesRendererInfo holds all data series specific rendering information.
  /// </summary>
  internal class SeriesRendererInfo : RendererInfo
  {
    internal Series series;

    internal PointRendererInfo[] pointRendererInfos;

    internal LineFormat LineFormat;
    internal FillFormat Fillformat;

    // Used if ChartType is set to Line
    internal XUnit MarkerSize;
    internal MarkerStyle MarkerStyle;
    internal XColor MarkerForegroundColor;
    internal XColor MarkerBackgroundColor;
  }

  /// <summary>
  /// An AxisRendererInfo holds all axis specific rendering information.
  /// </summary>
  internal class AxisRendererInfo : AreaRendererInfo
  {
    internal Axis axis;

    internal double MinimumScale;
    internal double MaximumScale;
    internal double MajorTick;
    internal double MinorTick;
    internal double MajorTickMarkWidth;
    internal double MinorTickMarkWidth;

    //Gridlines
    internal LineFormat MajorGridlinesLineFormat;
    internal LineFormat MinorGridlinesLineFormat;

    //AxisTitle
    internal XFont AxisTitleFont;
    internal XBrush AxisTitleBrush;
    internal XUnit AxisTitleOrientation;
    internal HorizontalAlignment AxisTitleAlignment;
    internal XSize AxisTitleSize;

    //TickLabels
    internal string TickLabelsFormat;
    internal XFont TickLabelsFont;
    internal XBrush TickLabelsBrush;
    internal double TickLabelsHeight;

    //LineFormat
    internal LineFormat LineFormat;

    //Chart.XValues, used for X axis only.
    internal XValues XValues;
  }

  /// <summary>
  /// Stores rendering information common to all plot area renderers.
  /// </summary>
  internal class PlotAreaRendererInfo : AreaRendererInfo
  {
    internal PlotArea plotArea;

    internal LineFormat LineFormat;
  }

}
